'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _vendorSticky = require('./vendorSticky');

var _vendorSticky2 = _interopRequireDefault(_vendorSticky);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var SCROLL_REASON = {
    OBSERVED: 'observed',
    REQUESTED: 'requested'
};

var StickyTree = function (_React$PureComponent) {
    _inherits(StickyTree, _React$PureComponent);

    function StickyTree(props) {
        _classCallCheck(this, StickyTree);

        var _this = _possibleConstructorReturn(this, (StickyTree.__proto__ || Object.getPrototypeOf(StickyTree)).call(this, props));

        _this.onScroll = _this.onScroll.bind(_this);

        _this.state = {
            scrollTop: 0,
            currNodePos: 0,
            // used to know when an update was caused by a scroll so that we don't unnecessarily re-render.
            scrollTick: false
        };

        _this.nodePosCache = [];
        _this.rowRenderRange = undefined;
        return _this;
    }

    /**
     *  Converts the consumer's tree structure into a flat array with root at index: 0,
     *  including information about the top and height of each node.
     *
     *  i.e:
     *  [
     *    { id: 'root', top: 0, index: 0, height: 100. isSticky: true , zIndex: 0, stickyTop: 10 },
     *    { id: 'child1', top: 10, index: 1, parentIndex: 0 height: 10, isSticky: false },
     *    ...
     *  ]
     */


    _createClass(StickyTree, [{
        key: 'flattenTree',
        value: function () {
            function flattenTree(node) {
                var props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.props;
                var nodes = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
                var isFirstChild = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
                var isLastChild = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;
                var parentIndex = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : undefined;
                var context = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : { totalHeight: 0 };

                var index = nodes.length;
                var height = node.height !== undefined ? node.height : props.defaultRowHeight;

                var nodeInfo = {
                    id: node.id,
                    isSticky: node.isSticky || false,
                    stickyTop: node.stickyTop || 0,
                    zIndex: node.zIndex || 0,
                    top: context.totalHeight,
                    parentIndex: parentIndex,
                    parentInfo: nodes[parentIndex],
                    height: height,
                    index: index,
                    isFirstChild: isFirstChild,
                    isLastChild: isLastChild
                };

                nodes.push(nodeInfo);

                if (parentIndex !== undefined) {
                    nodes[parentIndex].children.push(index);
                }

                context.totalHeight += height;

                var children = props.getChildren(node.id);
                if (Array.isArray(children)) {
                    nodeInfo.children = [];
                    for (var i = 0; i < children.length; i++) {
                        // Need to reset parentIndex here as we are recursive.
                        var child = children[i];
                        this.flattenTree(child, props, nodes, i === 0, i === children.length - 1, index, context);
                    }
                }

                nodeInfo.totalHeight = context.totalHeight - nodeInfo.top;

                return nodes;
            }

            return flattenTree;
        }()
    }, {
        key: 'componentWillMount',
        value: function () {
            function componentWillMount() {
                this.refreshCachedMetadata(this.props);
                this.storeRenderTree(this.props, this.state);
            }

            return componentWillMount;
        }()
    }, {
        key: 'hasStructureChanged',
        value: function () {
            function hasStructureChanged(newProps) {
                return newProps.root !== this.props.root || newProps.getChildren !== this.props.getChildren || newProps.defaultRowHeight !== this.props.defaultRowHeight;
            }

            return hasStructureChanged;
        }()
    }, {
        key: 'componentWillReceiveProps',
        value: function () {
            function componentWillReceiveProps(newProps) {
                // These two properties will change when the structure changes, so we need to re-build the tree when this happens.
                if (this.hasStructureChanged(newProps)) {
                    this.refreshCachedMetadata(newProps);
                }

                if (newProps.scrollIndex !== undefined && newProps.scrollIndex >= 0) {
                    this.scrollIndexIntoView(newProps.scrollIndex);
                }
            }

            return componentWillReceiveProps;
        }()
    }, {
        key: 'componentWillUpdate',
        value: function () {
            function componentWillUpdate(newProps, newState) {
                if (newState.scrollTick === this.state.scrollTick || newState.currNodePos !== this.state.currNodePos) {
                    this.storeRenderTree(newProps, newState);
                }
            }

            return componentWillUpdate;
        }()

        /**
         * Returns the index of the node in a flat list tree (post-order traversal).
         *
         * @param nodeId The node index to get the index for.
         * @returns {number}
         */

    }, {
        key: 'getNodeIndex',
        value: function () {
            function getNodeIndex(nodeId) {
                // TODO: Might be best to create a lookup to support this.
                for (var i = 0, l = this.nodePosCache.length; i < l; ++i) {
                    if (this.nodePosCache[i].id === nodeId) {
                        return i;
                    }
                }
                return -1;
            }

            return getNodeIndex;
        }()

        /**
         * Returns true if the node is completely visible and is not obscured.
         * This will return false when the node is partially obscured.
         *
         * @param nodeId The id of the node to check
         * @param includeObscured if true, this method will return true for partially visible nodes.
         * @returns {boolean}
         */

    }, {
        key: 'isNodeVisible',
        value: function () {
            function isNodeVisible(nodeId) {
                var includeObscured = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

                return this.isIndexVisible(this.getNodeIndex(nodeId), includeObscured);
            }

            return isNodeVisible;
        }()

        /**
         * Returns true if the node is completely visible and is not obscured.
         * This will return false when the node is partially obscured.
         *
         * @param index The index of the node to check, generally retrieved via getNodeIndex()
         * @param includeObscured if true, this method will return true for partially visible nodes.
         * @returns {boolean}
         */

    }, {
        key: 'isIndexVisible',
        value: function () {
            function isIndexVisible(index) {
                var includeObscured = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

                var inView = void 0;
                var node = this.nodePosCache[index];
                if (!includeObscured) {
                    inView = this.isIndexInViewport(index);
                } else {
                    inView = this.elem.scrollTop <= node.top + node.height - node.stickyTop && this.elem.scrollTop + this.props.height >= node.top;
                }
                if (inView) {
                    // If this node is in view, new need to check to see if it is obscured by a sticky parent.
                    // Note that this does not handle weird scenarios where the node's parent has a sticky top which is less than other ancestors.
                    // Or any z-index weirdness.
                    var path = this.getParentPath(index, false);
                    for (var i = 0; i < path.length; i++) {
                        var ancestor = path[i];
                        // If the ancestor is sticky and the node is in view, then it must be stuck to the top
                        if (ancestor.isSticky) {
                            if (!includeObscured && ancestor.stickyTop + ancestor.height > node.top - this.elem.scrollTop) {
                                return false;
                            }
                            if (includeObscured && ancestor.stickyTop + ancestor.height > node.top + node.height - this.elem.scrollTop) {
                                return false;
                            }
                        }
                    }
                    return true;
                }
                return false;
            }

            return isIndexVisible;
        }()

        /**
         * Returns true if the node is within the view port window. Note this this will return FALSE for visible sticky nodes that are
         * partially out of view disregarding sticky, which is useful when the node will become unstuck. This may occur when the node is
         * collapsed in a tree. In this case, you want to scroll this node back into view so that the collapsed node stays in the same position.
         *
         * @param nodeId The id of the node to check
         * @returns {boolean}
         */

    }, {
        key: 'isNodeInViewport',
        value: function () {
            function isNodeInViewport(nodeId) {
                return this.isIndexInViewport(this.getNodeIndex(nodeId));
            }

            return isNodeInViewport;
        }()

        /**
         * Returns true if the node is within the view port window. Note this this will return FALSE for visible sticky nodes that are
         * partially out of view disregarding sticky, which is useful when the node will become unstuck. This may occur when the node is
         * collapsed in a tree. In this case, you want to scroll this node back into view so that the collapsed node stays in the same position.
         *
         * This also returns false if the node is partially out of view.
         *
         * @param index The node index, generally retrieved via getNodeIndex()
         * @returns {boolean}
         */

    }, {
        key: 'isIndexInViewport',
        value: function () {
            function isIndexInViewport(index) {
                var node = this.nodePosCache[index];
                return this.elem.scrollTop <= node.top - node.stickyTop && this.elem.scrollTop + this.props.height >= node.top + node.height;
            }

            return isIndexInViewport;
        }()

        /**
         * Returns the top of the node with the specified id.
         * @param nodeId
         */

    }, {
        key: 'getNodeTop',
        value: function () {
            function getNodeTop(nodeId) {
                return this.getIndexTop(this.getNodeIndex(nodeId));
            }

            return getNodeTop;
        }()

        /**
         * Returns the top of the node with the specified index.
         * @param index
         */

    }, {
        key: 'getIndexTop',
        value: function () {
            function getIndexTop(index) {
                return this.nodePosCache[index].top;
            }

            return getIndexTop;
        }()

        /**
         * Returns the scrollTop of the scrollable element
         */

    }, {
        key: 'getScrollTop',
        value: function () {
            function getScrollTop() {
                return this.elem.scrollTop;
            }

            return getScrollTop;
        }()

        /**
         * Sets the scrollTop position of the scrollable element.
         * @param scrollTop
         */

    }, {
        key: 'setScrollTop',
        value: function () {
            function setScrollTop(scrollTop) {
                if (!isNaN(scrollTop)) {
                    this.setState({ scrollTop: scrollTop, scrollReason: SCROLL_REASON.REQUESTED });
                }
            }

            return setScrollTop;
        }()

        /**
         * Scrolls the node into view so that it is visible.
         *
         * @param nodeId The node id of the node to scroll into view.
         * @param alignToTop if true, the node will aligned to the top of viewport, or sticky parent. If false, the bottom of the node will
         * be aligned with the bottom of the viewport.
         */

    }, {
        key: 'scrollNodeIntoView',
        value: function () {
            function scrollNodeIntoView(nodeId) {
                var alignToTop = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

                this.scrollIndexIntoView(this.getNodeIndex(nodeId), alignToTop);
            }

            return scrollNodeIntoView;
        }()

        /**
         * Scrolls the node into view so that it is visible.
         *
         * @param index The index of the node.
         * @param alignToTop if true, the node will aligned to the top of viewport, or sticky parent. If false, the bottom of the node will
         * be aligned with the bottom of the viewport.
         */

    }, {
        key: 'scrollIndexIntoView',
        value: function () {
            function scrollIndexIntoView(index) {
                var alignToTop = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

                var node = this.nodePosCache[index];
                if (node !== undefined) {
                    var scrollTop = void 0;
                    if (alignToTop) {
                        if (node.isSticky) {
                            scrollTop = node.top - node.stickyTop;
                        } else {
                            var path = this.getParentPath(index, false);
                            for (var i = 0; i < path.length; i++) {
                                var ancestor = path[i];
                                if (ancestor.isSticky) {
                                    scrollTop = node.top - ancestor.stickyTop - ancestor.height;
                                    break;
                                }
                            }
                            if (scrollTop === undefined) {
                                // Fallback if nothing is sticky.
                                scrollTop = node.top;
                            }
                        }
                    } else {
                        scrollTop = node.top - this.props.height + node.height;
                    }
                    this.setScrollTop(scrollTop);
                }
            }

            return scrollIndexIntoView;
        }()
    }, {
        key: 'componentDidUpdate',
        value: function () {
            function componentDidUpdate(prevProps, prevState) {
                if (this.state.scrollReason === SCROLL_REASON.REQUESTED) {
                    if (this.state.scrollTop >= 0 && this.state.scrollTop !== this.elem.scrollTop) {
                        this.elem.scrollTop = this.state.scrollTop;
                    }
                }

                if (this.props.onRowsRendered !== undefined && (prevState.currNodePos !== this.state.currNodePos || this.hasStructureChanged(prevProps))) {
                    var range = this.rowRenderRange;
                    var visibleStartInfo = this.nodePosCache[range.visibleStart];
                    var visibleEndInfo = this.nodePosCache[range.visibleEnd];

                    this.props.onRowsRendered({
                        overscanStartIndex: range.start,
                        overscanStopIndex: range.end,
                        startIndex: range.visibleStart,
                        stopIndex: range.visibleEnd,
                        startNode: visibleStartInfo && visibleStartInfo.id,
                        endNode: visibleEndInfo && visibleEndInfo.id
                    });
                }
            }

            return componentDidUpdate;
        }()
    }, {
        key: 'refreshCachedMetadata',
        value: function () {
            function refreshCachedMetadata(props) {
                this.nodePosCache = this.flattenTree(props.root, props);
                // Need to re-render as the curr node may not be in view
                if (this.elem) {
                    // We need to find the the closest node to where we are scrolled to since the structure of the
                    // the tree probably has changed.
                    this.findClosestNode(this.elem.scrollTop, 0);
                }
            }

            return refreshCachedMetadata;
        }()
    }, {
        key: 'recomputeTree',
        value: function () {
            function recomputeTree() {
                if (this.props.root !== undefined && this.props.getChildren !== undefined) {
                    this.refreshCachedMetadata(this.props);
                    this.forceUpdate();
                }
            }

            return recomputeTree;
        }()
    }, {
        key: 'storeRenderTree',
        value: function () {
            function storeRenderTree(props, state) {
                this.treeToRender = this.renderParentTree(props, state);
            }

            return storeRenderTree;
        }()
    }, {
        key: 'renderParentTree',
        value: function () {
            function renderParentTree(props, state) {
                this.rowRenderRange = this.getRenderRowRange(props, state);
                var path = this.getParentPath(this.rowRenderRange.start);

                // Parent nodes to the current range.
                var indexesToRender = new Set();
                for (var i = 0; i < path.length; i++) {
                    indexesToRender.add(path[i].index);
                }

                // The rest of the nodes within the range.
                for (var _i = this.rowRenderRange.start; _i <= this.rowRenderRange.end; _i++) {
                    indexesToRender.add(this.nodePosCache[_i].index);
                }

                if (this.props.renderRoot) {
                    return _react2['default'].createElement(
                        'div',
                        { className: 'rv-sticky-node-list', style: { width: '100%', position: 'absolute', top: 0 } },
                        this.renderChildWithChildren(props, state, this.nodePosCache[0], 0, indexesToRender)
                    );
                }
                return this.renderParentContainer(props, state, this.nodePosCache[0], indexesToRender);
            }

            return renderParentTree;
        }()
    }, {
        key: 'renderParentContainer',
        value: function () {
            function renderParentContainer(props, state, parent, indexesToRender) {
                return _react2['default'].createElement(
                    'div',
                    {
                        className: 'rv-sticky-node-list',
                        style: { position: 'absolute', width: '100%', height: parent.totalHeight - parent.height }
                    },
                    this.renderChildren(props, state, parent, indexesToRender)
                );
            }

            return renderParentContainer;
        }()
    }, {
        key: 'getChildContainerStyle',
        value: function () {
            function getChildContainerStyle(child, top) {
                return { position: 'absolute', top: top, height: child.totalHeight, width: '100%' };
            }

            return getChildContainerStyle;
        }()
    }, {
        key: 'renderChildWithChildren',
        value: function () {
            function renderChildWithChildren(props, state, child, top, indexesToRender) {
                return _react2['default'].createElement(
                    'div',
                    { key: 'rv-node-' + child.id, className: 'rv-sticky-parent-node',
                        style: this.getChildContainerStyle(child, top) },
                    this.renderNode(props, state, child, this.getClientNodeStyle(child)),
                    this.renderParentContainer(props, state, child, indexesToRender)
                );
            }

            return renderChildWithChildren;
        }()
    }, {
        key: 'getClientNodeStyle',
        value: function () {
            function getClientNodeStyle(node) {
                var style = { height: node.height };
                if (node.isSticky) {
                    style.position = (0, _vendorSticky2['default'])();
                    style.top = node.stickyTop;
                    style.zIndex = node.zIndex;
                }

                return style;
            }

            return getClientNodeStyle;
        }()
    }, {
        key: 'getClientLeafNodeStyle',
        value: function () {
            function getClientLeafNodeStyle(node, top) {
                return {
                    position: 'absolute',
                    top: top,
                    height: node.height,
                    width: '100%'
                };
            }

            return getClientLeafNodeStyle;
        }()
    }, {
        key: 'renderChildren',
        value: function () {
            function renderChildren(props, state, parent, indexesToRender) {
                var _this2 = this;

                var nodes = [];
                var top = 0;
                parent.children.forEach(function (index) {
                    var child = _this2.nodePosCache[index];

                    if (indexesToRender.has(index)) {
                        if (child.children && child.children.length > 0) {
                            nodes.push(_this2.renderChildWithChildren(props, state, child, top, indexesToRender));
                        } else {
                            // Sticky nodes will need a container so that their top is correct. The sticky node itself will have a top
                            // of the offset where it should stick, which would conflict with the absolute position of the node.
                            if (child.isSticky || props.wrapAllLeafNodes) {
                                nodes.push(_react2['default'].createElement(
                                    'div',
                                    {
                                        className: 'rv-sticky-leaf-node',
                                        key: 'rv-node-' + child.id,
                                        style: _this2.getChildContainerStyle(child, top) },
                                    _this2.renderNode(props, state, child, _this2.getClientNodeStyle(child))
                                ));
                            } else {
                                nodes.push(_this2.renderNode(props, state, child, _this2.getClientLeafNodeStyle(child, top)));
                            }
                        }
                    }
                    // Needs to be on the outside so that we add the the top even if
                    // this node is not visible
                    top += child.totalHeight;
                });
                return nodes;
            }

            return renderChildren;
        }()
    }, {
        key: 'renderNode',
        value: function () {
            function renderNode(props, state, nodeInfo, style) {
                return props.rowRenderer({ id: nodeInfo.id, nodeInfo: nodeInfo, style: style });
            }

            return renderNode;
        }()

        /**
         * Determines the start and end number of the range to be rendered.
         * @returns {{start: number, end: number}} Indexes within nodePosCache
         */

    }, {
        key: 'getRenderRowRange',
        value: function () {
            function getRenderRowRange(props, state) {
                // Needs to be at least 1
                var overscanRowCount = props.overscanRowCount > 0 ? props.overscanRowCount : 1;
                var start = state.currNodePos - overscanRowCount;
                if (start < 0) {
                    start = 0;
                }
                var visibleEnd = state.currNodePos + 1;

                while (this.nodePosCache[visibleEnd] && this.nodePosCache[visibleEnd].top < state.scrollTop + props.height) {
                    visibleEnd++;
                }

                var end = visibleEnd + overscanRowCount;
                if (end > this.nodePosCache.length - 1) {
                    end = this.nodePosCache.length - 1;
                }

                return { start: start, end: end, visibleStart: state.currNodePos, visibleEnd: visibleEnd };
            }

            return getRenderRowRange;
        }()

        /**
         * Returns the parent path from nodePosCache for the specified index within nodePosCache.
         * @param nodeIndex
         * @param topDownOrder if true, the array with index 0 will be the root node, otherwise 0 will be the immediate parent.
         * @returns {Array<Node>}
         */

    }, {
        key: 'getParentPath',
        value: function () {
            function getParentPath(nodeIndex) {
                var topDownOrder = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

                var currNode = this.nodePosCache[nodeIndex];
                var path = [];
                while (currNode) {
                    currNode = this.nodePosCache[currNode.parentIndex];
                    if (currNode) {
                        path.push(currNode);
                    }
                }
                return topDownOrder ? path.reverse() : path;
            }

            return getParentPath;
        }()

        /**
         * Searches from the current node position downwards to see if the top of nodes above are greater
         * than or equal to the current scrollTop
         * @param scrollTop
         * @param searchPos
         * @returns {number}
         */

    }, {
        key: 'forwardSearch',
        value: function () {
            function forwardSearch(scrollTop, searchPos) {
                var nodePosCache = this.nodePosCache;
                for (var i = searchPos; i < nodePosCache.length; i++) {
                    if (nodePosCache[i].top >= scrollTop) {
                        return i;
                    }
                }
                return nodePosCache.length - 1;
            }

            return forwardSearch;
        }()

        /**
         * Searches from the current node position upwards to see if the top of nodes above are less than
         * or equal the current scrollTop.
         * @param scrollTop
         * @param searchPos
         * @returns {number}
         */

    }, {
        key: 'backwardSearch',
        value: function () {
            function backwardSearch(scrollTop, searchPos) {
                var nodePosCache = this.nodePosCache;
                for (var i = searchPos; i >= 0; i--) {
                    if (nodePosCache[i].top <= scrollTop) {
                        return i;
                    }
                }
                return 0;
            }

            return backwardSearch;
        }()

        /**
         * Returns the closest node within nodePosCache.
         * @param scrollTop
         * @param currNodePos
         */

    }, {
        key: 'findClosestNode',
        value: function () {
            function findClosestNode(scrollTop, currNodePos) {
                var pos = void 0;
                if (scrollTop > this.state.scrollTop || currNodePos === 0) {
                    pos = this.forwardSearch(scrollTop, currNodePos);
                }
                if (scrollTop < this.state.scrollTop || pos === undefined) {
                    pos = this.backwardSearch(scrollTop, currNodePos);
                }
                this.setState({ currNodePos: pos });
            }

            return findClosestNode;
        }()
    }, {
        key: 'onScroll',
        value: function () {
            function onScroll(e) {
                var _e$target = e.target,
                    scrollTop = _e$target.scrollTop,
                    scrollLeft = _e$target.scrollLeft;

                this.findClosestNode(scrollTop, this.state.currNodePos);

                this.setState({ scrollTop: scrollTop, scrollReason: SCROLL_REASON.OBSERVED, scrollTick: !this.state.scrollTick });

                if (this.props.onScroll !== undefined) {
                    this.props.onScroll({ scrollTop: scrollTop, scrollLeft: scrollLeft, scrollReason: SCROLL_REASON.OBSERVED });
                }
            }

            return onScroll;
        }()
    }, {
        key: 'render',
        value: function () {
            function render() {
                var _this3 = this;

                var style = { overflow: 'auto', position: 'relative' };
                if (this.props.width) {
                    style.width = this.props.width;
                }
                if (this.props.height) {
                    style.height = this.props.height;
                }

                return _react2['default'].createElement(
                    'div',
                    { ref: function () {
                            function ref(elem) {
                                return _this3.elem = elem;
                            }

                            return ref;
                        }(), className: 'rv-sticky-tree', style: style, onScroll: this.onScroll },
                    this.treeToRender
                );
            }

            return render;
        }()
    }]);

    return StickyTree;
}(_react2['default'].PureComponent);

StickyTree.propTypes = {

    /**
     * Returns an array of child objects that represent the children of a particular node.
     * The returned object for each child should be in the form:
     *
     * { id: 'childId', height: [number], isSticky: [true|false], stickyTop: [number], zIndex: 0 }
     *
     * Where id and height are mandatory. If isSticky is true, stickyTop and zIndex should also be returned.
     *
     * Example:
     *
     * const getChildren = (id) => {
     *    return myTree[id].children.map(childId => ({
     *      id: childId
     *      isSticky: nodeIsSticky(childId),
     *      height: myTree[childId].height,
     *      ...
     *    }))
     * }
     */
    getChildren: _propTypes2['default'].func.isRequired,

    /**
     * Called to retrieve a row to render. The function should return a single React node.
     * The function is called with an object in the form:
     *
     * <pre>
     *     rowRenderer({ id, style, nodeInfo })
     * </pre>
     *
     * The id is the id from either the root property passed to the tree, or one returned in the getChildren call.
     */
    rowRenderer: _propTypes2['default'].func.isRequired,

    /**
     * An object which represents the root node in the form:
     *
     * {
     *   id: 'myRootNodeId',
     *   isSticky: true
     * }
     *
     * This will be the first node passed to rowRenderer({id: myRootNodeId, ...}). Your id might be an index in an array or map lookup.
     */
    root: _propTypes2['default'].object.isRequired,

    /**
     * Lets StickyTree know how many rows above and below the visible area should be rendered, to improve performance.
     */
    overscanRowCount: _propTypes2['default'].number.isRequired,

    /**
     * The height of the outer container.
     */
    height: _propTypes2['default'].number,

    /**
     * The width of the outer container
     */
    width: _propTypes2['default'].number,

    /**
     * if true, the root node will be rendered (by calling rowRenderer() for the root id). Otherwise no root node will be rendered.
     */
    renderRoot: _propTypes2['default'].bool,

    /**
     * Sets the position of the tree to the specified scrollTop. To reset
     * this, change this to -1 or undefined
     */
    scrollTop: _propTypes2['default'].number,

    /**
     * Sets the position of the tree to the specified scrollIndex. This is useful when
     * paired with onRowsRendered() which returns the startIndex and stopIndex.
     */
    scrollIndex: _propTypes2['default'].number,

    /**
     * Called whenever the scrollTop position changes.
     */
    onScroll: _propTypes2['default'].func,

    /**
     * Called to indicate that a new render range for rows has been rendered.
     */
    onRowsRendered: _propTypes2['default'].func,

    /**
     * Specifies the default row height which will be used if the child or root object do not have a height specified.
     */
    defaultRowHeight: _propTypes2['default'].number,

    /**
     * If true, all leaf nodes will be wrapped with a div, even when they are not sticky. this may help with certain tree structures where you need a constant key
     * for the element so that it is not recreated when React dom diffing occurs.
     */
    wrapAllLeafNodes: _propTypes2['default'].bool
};
StickyTree.defaultProps = {
    overscanRowCount: 10,
    renderRoot: true,
    wrapAllLeafNodes: false
};
exports['default'] = StickyTree;